;================================================================================
;
;TIA-232 I/O BUFFER DEFINITIONS
;
;	
;	This file defines the circular buffers that are required to drive the
;	26C92 via interrupts.  Read all comments before changing any defini-
;	tions, as there are a number of dependencies that must be maintained.
;	
;
n_e32buf =n_c92ch*2            ;number of buffers...
;
;	
;	Two buffers must be allocated per TIA-232 channel, one for input (RxD) &
;	the other for output (TxD).
;	
;
s_e32buf =128                  ;buffer size in bytes...
;
;	
;	Many 65xx systems use a 256 byte buffer size, as it lends itself to very
;	simple indexing.  However, as the 2 clock rate is increased the buffer
;	size becomes less critical, due to the higher throughput brought about
;	with the faster clock.  During initial firmware development for my POC
;	unit, I stuck with a 256 byte buffer size to reduce the potential for
;	bugs in the code.  However, four buffers of that size eat up 1KB of RAM.
;
;	Once I had a working DUART driver, I experimented with a smaller buffer
;	size to determine what the effect would be on performance.  I found I
;	could go as low as 32 bytes & still maintain acceptable throughput.  As
;	I was not seriously cramped for memory space, I settled on a 128 byte
;	buffer size as a compromise.
;
;	Generally speaking, apparent TxD performance is more affected by buffer
;	size than RxD performance.  As the TxD buffer size is reduced, there is
;	an increased tendency for foreground blocking to occur during sustained
;	transmission, e.g., while doing a full screen paint.  Hence there is an
;	advantage to use a large TxD buffer.
;
;	Excessively reducing the size of the RxD buffer may cause incoming bytes
;	to be dropped.  Once a byte has been read from the RxD FIFO it must be
;	stored or discarded, the latter occurring if the buffer is full.  If the
;	foreground application is able to read & process bytes faster than they
;	arrive from the remote endpoint, then dropping will not occur.  As earl-
;	ier noted, this becomes less an issue as the 2 clock rate is increased.
;
;	The value in E32XMASK (defined in the file MASKS.ASM) determines the
;	effective buffer size, since it is ANDed with the current buffer index
;	to produce the effective buffer index.  Using a 128 byte buffer size,
;	E32XMASK is $7F, which causes the index to stay in the range 0 to 127.
;	If E32XMASK is $3F then the index would stay in the range 0 to 63, prod-
;	ucing an effective 64 byte buffer size, regardless of the actual size
;	that has been defined.  E32XMASK is derived from the buffer size defined
;	by S_E32BUF & will be correct as long as S_E32BUF == 2^N, where N is any
;	positive integer in the range of 2 to 8.
;
;	Something that could be done is to have separate sizes for the RxD & TxD
;	buffers, along with two index masks.  Doing so would allow the size def-
;	inition for the TxD buffer to be larger than that for the RxD buffer.  I
;	have not investigated the value of doing so.
;
;	Incidentally, a buffer size larger than 256 bytes is relatively easy to
;	implement with the 65C816, since it has 16 bit indexing.  However, it's
;	not likely that you'll see any benefit to doing so, & RAM consumption
;	will substantially increase.
;	
;
s_e32spc =s_e32buf*n_e32buf    ;total buffer space...
;
;	
;	S_E32SPC defines the total space in RAM occupied by the buffers.  You
;	should use this value to allocate space, since it is dependent on the
;	buffer size defined by S_E32BUF
;	
;
e32buffr =$00c000              ;start of buffers...
;
;	
;	E32BUFFR defines where in RAM the buffers will start.  All routines that
;	access these buffers do not change the DB register, so by default, buf-
;	fers are assumed to be in bank $00.  The interrupt service routine pres-
;	erves DB, so it is possible to define an alternate bank if desired.  You
;	would also have to tinker with the GETCHA, GETCHB, PUTCHA & PUTCHB func-
;	tions, as none changes DB.
;
;	Be sure to allocate enough RAM for all buffers.  Refer to S_E32SPC.
;	
;
e32bufta =e32buffr             ;ch A TxD buffer
e32bufra =e32bufta+s_e32buf    ;ch A RxD buffer
e32buftb =e32bufra+s_e32buf    ;ch B TxD buffer
e32bufrb =e32buftb+s_e32buf    ;ch B RxD buffer
;
	.end
