;================================================================================
;
;	
;	This file contains code for the foreground processing part of the 26C92
;	driver.  The functions herein are:
;
;	GETCHA    Gets datum from channel A.
;	GETCHB    Gets datum from channel B.
;	PUTCHA    Puts datum to channel A.
;	PUTCHA    Puts datum to channel A.
;	CHANBCTL  Enables or disables channel B.
;
;	In the above, "datum" is synonomous with "byte."  Each function has com-
;	mentary that describes the calling procedure & returns.
;
;	The GET... functions read their respective channels' buffers, returning
;	immediately regardless of the availability of data.  Data availability
;	is indicated by the MPU's carry bit, which must be tested after each ac-
;	cess to determine if a datum was returned.
;
;	The PUT... functions write their respective channels' buffers & will not
;	return until the datum has been accepted & buffered.  Blocking will oc-
;	cur if the buffer is full & the interrupt part of the driver cannot load
;	any more data into the 26C92's transmit FIFO.  The most likely reason
;	for this situation is that the remote device has stopped transmission by
;	deasserting the channel's clear-to-send (CTS) signal.  Blocking will al-
;	so occur if IRQs have been disabled, as the interrupt part of the driver
;	will no longer be executed.
;
;	The CHANBCTL ("channel B control") function may be used to enable or
;	disable channel B's receiver as required.  26C92 setup doesn't enable
;	this channel's receiver.  The transmitter is automatically disabled when
;	the buffer is empty.
;	
;
;================================================================================
;
;getcha: GET DATUM FROM TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Example: JSR GETCHA
;	         BCS NODATUM
;	         STA DATUM
;	
;
getcha   shorta                ;8 bit accumulator
         sec                   ;assume no datum
         php                   ;save register widths
         longx                 ;16 bit index saves
         phx                   ;save
         phy                   ;save
         shortx                ;8 bit registers
;
;
.yreg    =1
.xreg    =.yreg+s_word
.sreg    =.xreg+s_word
;
;
         ldx e32rixga          ;"get" index
         cpx e32rixpa          ;"put" index
         beq getchaab          ;buffer empty
;
         ldy e32bufra,x        ;get datum
         txa                   ;old "get" index
         ina                   ;adjust
         and #e32xmask         ;deal with wrap
         sta e32rixga          ;new "get" index
         ldas .sreg            ;get SR stack copy
         and #sr_car_i         ;clear carry to...
         stas .sreg            ;indicate datum available
;
getchaaa tya                   ;recover datum
;
getchaab longx                 ;16 bit index loads
         ply                   ;restore
         plx                   ;restore
         plp                   ;restore
         rts
;
;================================================================================
;
;getchb: GET DATUM FROM TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Notes: 1) See code comments in GETCHA for
;	          execution details.
;
;	Example: JSR GETCHB
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchb   shorta
         sec
         php
         longx
         phx
         phy
         shortr
;
;
.yreg    =1
.xreg    =.yreg+s_word
.sreg    =.xreg+s_word
;
;
         ldx e32rixgb
         cpx e32rixpb
         beq getchaab
;
         ldy e32bufrb,x
         txa
         ina
         and #e32xmask
         sta e32rixgb
         ldas .sreg
         and #sr_car_i
         stas .sreg
         bra getchaaa
;
;================================================================================
;
;putcha: PUT DATUM TO TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           +++++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHA
;
;	Notes: 1) This function will block if the buffer is full.
;	
;
putcha   clc                   ;never an error
         php                   ;save MPU state
         longr
         pha                   ;save all registers
         phx
         phy
         cli                   ;IRQs must be enabled
         shortr                ;8 bit registers
         tay                   ;buffer datum
         ldx e32tixpa          ;buffer "put" index
         txa                   ;make a copy
         ina                   ;point to next position
         and #e32xmask         ;deal with wrap
;
.0000010 cmp e32tixga          ;buffer "get" index
         bne .0000020          ;buffer not full
;
         wai                   ;waste some time &...
         bra .0000010          ;try again
;
.0000020 xba                   ;hold new index
         tya                   ;recover datum &...
         sta e32bufta,x        ;buffer it
         xba                   ;recover new index &...
         sta e32tixpa          ;set it
         bit e32txst           ;transmitter enabled?
         bpl putchaa           ;yes
;
         lda #e32txsta
         trb e32txst           ;clear disable flag &...
         lda #u92crtxe         ;enable...
         sta ccr_92a           ;transmitter
;
putchaa  longr
         ply                   ;clean up stack
         plx
         pla
         plp
         rts
;
;================================================================================
;
;putchb: PUT DATUM TO TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           +++++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHB
;
;	Notes: 1) This function will block if the buffer is full.
;	       2) See code comments in PUTCHA for execution
;	          details.
;	
;
putchb   clc
         php
         longr
         pha
         phx
         phy
         cli
         shortr
         tay
         ldx e32tixpb
         txa
         ina
         and #e32xmask
;
.0000010 cmp e32tixgb
         bne .0000020
;
         wai
         bra .0000010
;
.0000020 xba
         tya
         sta e32buftb,x
         xba
         sta e32tixpb
         bit e32txst
         bvc putchaa
;
         lda #e32txstb
         trb e32txst
         lda #u92crtxe
         sta ccr_92b
         bra putchaa
;
;================================================================================
;
;chanbctl: TIA-232 CHANNEL B RECEIVER CONTROL
;
;	
;	Preparatory Ops : see programming example
;
;	Register Returns: .A: used
;	                  .B: entry value
;	                  .X: entry value
;	                  .Y: entry value
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> entry value
;	           ||||||+> 0
;	           |||+++> entry values
;	           ||+> 1
;	           |+> entry value
;	           +> undefined
;
;	Examples: clc              ;disable
;	          jsr chanbctl
;
;	          sec              ;enable
;	          jsr chanbctl
;	
;
chanbctl shorta
         bcc disabchb          ;disable
;
;================================================================================
;
;enabchb: ENABLE DUART CHANNEL B RECEIVER
;
enabchb  lda #u92crrsa|u92crrxe ;enable RxD & assert RTS
         bra chbctl
;
;================================================================================
;
;disabchb: DISABLE DUART CHANNEL B RECEIVER
;
disabchb lda #u92crrsd|u92crrxd ;disable RxD & deassert RTS
;
;================================================================================
;
;chbctl: DUART CHANNEL B CONTROL
;
chbctl   sta ccr_92b           ;write to command register
         wai                   ;DUART reaction delay
         rts
;
	.end
